/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.pvm.log;

import java.io.Serializable;
import java.util.Date;

import org.jbpm.pvm.Execution;
import org.jbpm.pvm.impl.ExecutionImpl;
import org.jbpm.util.Clock;

public abstract class ProcessLog implements Serializable {
  
  private static final long serialVersionUID = 1L;

  long dbid = 0;
  protected int index = -1;
  protected Date time = null;
  protected Execution execution = null;
  
  public ProcessLog() {
  }

  public void setExecution(ExecutionImpl execution) {
    this.execution = execution;
    this.time = Clock.getCurrentTime();
    this.index = execution.nextLogIndex();
  }

  /**
   * provides a text description for this update
   * which can be used e.g. in the admin web console.
   */
  public abstract String toString();
  
  public long getDbid() {
    return dbid;
  }
  public Date getTime() {
    return time;
  }
  public void setTime(Date date) {
    this.time = date;
  }
  public Execution getExecution() {
    return execution;
  }
  public int getIndex() {
    return index;
  }
}
